package com.ardublock.translator.block.didacticnetwork;

import com.ardublock.translator.Translator;
import com.ardublock.translator.block.TranslatorBlock;
import com.ardublock.translator.block.exception.SocketNullException;
import com.ardublock.translator.block.exception.SubroutineNotDeclaredException;

import edu.mit.blocks.codeblocks.BlockGenus;
import edu.mit.blocks.workspace.Workspace;

public class PsnClientBlock extends TranslatorBlock
{
	private Long myBlockID = 0L;
	
	public PsnClientBlock(Long blockId, Translator translator, String codePrefix, String codeSuffix, String label)
	{
		super(blockId, translator, codePrefix, codeSuffix, label);
		
		myBlockID = blockId;
	}

	@Override
	public String toCode() throws SocketNullException, SubroutineNotDeclaredException
	{
		String serialName = "sSerial";
		//String pubSubName = "";
		String callbackName = "";
		String pubSubType = "DidacticPSNetClient";
		
		boolean basicBlock = this.getTranslator().getBlock(myBlockID).getGenusName().toUpperCase().contains("BASIC");
		
		//Label autoText set in edu.mit.blocks.renderable.BlockLabel -> generateLabelText()
		String pubSubName = label.trim();
		if(pubSubName.contains(" ")) {
			pubSubName = pubSubName.substring(pubSubName.lastIndexOf(" ")+1, pubSubName.length());
		}
		
		//TranslatorBlock psnName = this.getRequiredTranslatorBlockAtSocket(0);//Name
		TranslatorBlock rxPin = this.getRequiredTranslatorBlockAtSocket(0);//Pin Rx
		TranslatorBlock txPin = this.getRequiredTranslatorBlockAtSocket(1);//Pin Tx
		
		translator.addHeaderFile("DidacticNet.h");
		translator.addHeaderFile("SoftwareSerial.h");
		
		translator.addDefinitionCommand("SoftwareSerial " + serialName+"(" + rxPin.toCode().replaceAll("\\s*_.new\\b\\s*", "") + ", "+ txPin.toCode().replaceAll("\\s*_.new\\b\\s*", "") +");\n");
		translator.addDefinitionCommand(pubSubType + " " +pubSubName+";\n");
		
		translator.addSetupCommand(serialName+".begin(2400);");
		
		if(!basicBlock) {
			TranslatorBlock cbName = this.getRequiredTranslatorBlockAtSocket(2);//Callback function
			callbackName = cbName.toCode().replaceAll("\\s*_.new\\b\\s*\"", "").replaceAll("\"", "");
			translator.addSetupCommand(pubSubName + ".begin(" + serialName +", "+ callbackName +");\n");	
		}
		else {
			translator.addSetupCommand(pubSubName + ".begin(" + serialName+");\n");	
		}
			
		return "";
	}
}
