
package com.ardublock.translator.block.output;

import com.ardublock.translator.Translator;
import com.ardublock.translator.block.TranslatorBlock;
import com.ardublock.translator.block.exception.BlockException;
import com.ardublock.translator.block.exception.SocketNullException;
import com.ardublock.translator.block.exception.SubroutineNotDeclaredException;
import com.ardublock.translator.block.numbers.ConstantNumberBlock;
import com.ardublock.translator.block.numbers.LocalVariableNumberBlock;
import com.ardublock.translator.block.numbers.NumberBlock;
import com.ardublock.translator.block.numbers.VariableNumberBlock;

public class ServoDefaultAttachBlock extends TranslatorBlock {

	public ServoDefaultAttachBlock(Long blockId, Translator translator, String codePrefix, String codeSuffix, String label)
	{
		super(blockId, translator, codePrefix, codeSuffix, label);
	}

	@Override
	public String toCode() throws SocketNullException, SubroutineNotDeclaredException
	{
		TranslatorBlock tb = this.getRequiredTranslatorBlockAtSocket(0);

		String servoSpecs = "";

		if (!( tb instanceof NumberBlock ) && !(tb instanceof VariableNumberBlock) && !(tb  instanceof LocalVariableNumberBlock) && !(tb instanceof ConstantNumberBlock)  )
		{
			throw new BlockException(this.blockId, "the Pin# of Servo must contain a number");
		}
		
		String pinNumber = tb.toCode().replaceAll("\\s*_.new\\b\\s*", "");
		
		int servoCounter = 1;
		while(translator.containsServo("servo"+servoCounter)) {
			servoCounter++;
		}
		
		String servoName = 	"servo"+servoCounter;
		translator.addServo("servo_"+pinNumber, servoName);
		
		//String servoName = "servo_pin_" + pinNumber;

	//	tb = this.getRequiredTranslatorBlockAtSocket(1);

		String ret = servoName + ".attach(" + pinNumber + servoSpecs + ");";
		translator.addHeaderFile("Servo.h");
		translator.addDefinitionCommand("Servo " + servoName + ";");
		//translator.addSetupCommand(servoName + ".detach(" + pinNumber + servoSpecs + ");");
		return ret;
	}

}
